// SPDX-License-Identifier: GPL-3.0-or-later
// Copyright (C) 2025 Abel Luck <abel@guardianproject.info>

package provider

import (
	"encoding/base64"
	"encoding/hex"
	"testing"
)

// Test data from a secret key generated by Tor version 0.4.9.2-alpha
// To reproduce:
//  1. Run: tor --keygen-family MyKey
//  2. This creates MyKey.secret_family_key and MyKey.public_family_id
//  3. Convert secret key to hex: xxd -p MyKey.secret_family_key | tr -d '\n'
const torGeneratedSecretKeyHex = "3d3d206564323535313976312d7365637265743a20666d6c792d6964203d3d00f84c620e227fcc5085eb538a29a11ac25abb052b6a36ddae008b307cca67fe792ecd73a67c0a7a28b2b747be2a59e5ef0c155e33217add6dac42dbc6f85a5162"
const torGeneratedPublicFamilyID = "xiHyFHgSU4/aaqPfRCrUk61jVBDQAv1hlHQd7lKIlI8"

func TestTorFamilyIdentity_SerializationAgainstTorGenerated(t *testing.T) {
	secretKeyBytes, err := hex.DecodeString(torGeneratedSecretKeyHex)
	if err != nil {
		t.Fatalf("Failed to decode secret key hex: %v", err)
	}

	const expectedHeader = "== ed25519v1-secret: fmly-id ==\x00"
	if len(secretKeyBytes) < len(expectedHeader) {
		t.Fatalf("Secret key file too short: %d bytes", len(secretKeyBytes))
	}

	header := string(secretKeyBytes[:len(expectedHeader)])
	if header != expectedHeader {
		t.Fatalf("Invalid header: got %q, want %q", header, expectedHeader)
	}

	expandedSK := secretKeyBytes[len(expectedHeader):]
	if len(expandedSK) != 64 {
		t.Fatalf("Invalid expanded secret key length: got %d, want 64", len(expandedSK))
	}

	publicKey, err := torComputePublicKey(expandedSK)
	if err != nil {
		t.Fatalf("Failed to compute public key: %v", err)
	}

	computedFamilyID := base64.RawStdEncoding.EncodeToString(publicKey)

	if computedFamilyID != torGeneratedPublicFamilyID {
		t.Errorf("Family ID mismatch:\n  computed: %s\n  expected: %s",
			computedFamilyID, torGeneratedPublicFamilyID)
	}
}

func TestTorFamilyIdentity_ExpandSeed(t *testing.T) {
	// Test that our seed expansion produces the same result as Tor
	// We'll need to reverse-engineer the seed from the expanded key
	// This is a secondary test to verify our torExpandSeed function
	secretKeyBytes, err := hex.DecodeString(torGeneratedSecretKeyHex)
	if err != nil {
		t.Fatalf("Failed to decode secret key hex: %v", err)
	}

	const expectedHeader = "== ed25519v1-secret: fmly-id ==\x00"
	expandedSK := secretKeyBytes[len(expectedHeader):]

	// Verify the key is properly clamped according to ed25519 standards
	// For ed25519, the clamping is:
	// - bits 0,1,2 of first byte cleared (make it a multiple of 8)
	// - bit 6 of byte 31 set
	// - bit 7 of byte 31 cleared
	if expandedSK[0]&0x07 != 0 {
		t.Errorf("First byte not properly clamped: %02x (bits 0,1,2 should be cleared)", expandedSK[0])
	}
	if expandedSK[31]&0x40 == 0 {
		t.Errorf("Byte 31 bit 6 should be set: %02x", expandedSK[31])
	}
	if expandedSK[31]&0x80 != 0 {
		t.Errorf("Byte 31 bit 7 should be cleared: %02x", expandedSK[31])
	}
}
