// SPDX-License-Identifier: GPL-3.0-or-later
// Copyright (c) 2025 Abel Luck <abel@guardianproject.info>

package provider

import (
	"encoding/base64"
	"fmt"
	"regexp"
	"strings"
	"testing"

	"github.com/hashicorp/terraform-plugin-testing/helper/resource"
	"github.com/hashicorp/terraform-plugin-testing/terraform"
)

func TestAccTorFamilyIdentityResource(t *testing.T) {
	resource.Test(t, resource.TestCase{
		PreCheck:                 func() { testAccPreCheck(t) },
		ProtoV6ProviderFactories: testAccProtoV6ProviderFactories,
		Steps: []resource.TestStep{
			{
				Config: testAccTorFamilyIdentityResourceConfig("MyFamily"),
				Check: resource.ComposeAggregateTestCheckFunc(
					resource.TestCheckResourceAttr("tor_family_identity.test", "family_name", "MyFamily"),
					resource.TestMatchResourceAttr("tor_family_identity.test", "id", regexp.MustCompile(`^[A-Za-z0-9+/]{43}$`)),
					resource.TestCheckResourceAttrSet("tor_family_identity.test", "secret_key"),
					testAccCheckTorFamilyIdentitySecretKeyFormat("tor_family_identity.test", "secret_key"),
				),
			},
		},
	})
}

func testAccCheckTorFamilyIdentitySecretKeyFormat(resourceName, attributeName string) resource.TestCheckFunc {
	return func(s *terraform.State) error {
		rs, ok := s.RootModule().Resources[resourceName]
		if !ok {
			return fmt.Errorf("Resource not found: %s", resourceName)
		}

		secretKeyBase64 := rs.Primary.Attributes[attributeName]
		if secretKeyBase64 == "" {
			return fmt.Errorf("secret_key attribute is empty")
		}

		secretKey, err := base64.StdEncoding.DecodeString(secretKeyBase64)
		if err != nil {
			return fmt.Errorf("Failed to decode secret_key: %v", err)
		}

		if len(secretKey) != 96 {
			return fmt.Errorf("secret_key has wrong size: expected 96 bytes, got %d", len(secretKey))
		}

		expectedHeader := "== ed25519v1-secret: fmly-id =="
		actualHeader := string(secretKey[:32])
		if !strings.HasPrefix(actualHeader, expectedHeader) {
			return fmt.Errorf("secret_key has wrong header: expected %q, got %q", expectedHeader, actualHeader[:len(expectedHeader)])
		}

		if secretKey[31] != 0 {
			return fmt.Errorf("secret_key header should end with null byte")
		}

		id := rs.Primary.Attributes["id"]
		if _, err := base64.RawStdEncoding.DecodeString(id); err != nil {
			return fmt.Errorf("id should be a valid base64 string without padding: %s", id)
		}

		if strings.HasSuffix(id, "=") {
			return fmt.Errorf("id should not have padding characters: %s", id)
		}

		return nil
	}
}

func testAccTorFamilyIdentityResourceConfig(familyName string) string {
	return fmt.Sprintf(`
resource "tor_family_identity" "test" {
  family_name = %[1]q
}
`, familyName)
}
